local gpadinfo = include('gpadinfo.lua');

Controller = class.new();

function Controller:constructor(profName, gpadType)
	self.type	=	'PS2';
	self.profile = {};

	if( profName ) then
		self:setProfile(profName)
	end

	if( gpadType ) then
		self:setType(gpadType)
	end
end

function Controller:setProfile(profName)
	local path = 'profiles/' .. profName .. '.lua';
	printf("Loading profile %s\n", path);
	self.profile = include(path, true);
end

function Controller:setType(gpadType)
	gpadType = string.upper(gpadType);
	if( not gpadinfo[gpadType] ) then
		error(sprintf("Gamepad type (%s) does not appear to be supported.", gpadType), 2);
	end

	printf("Setting gamepad type to %s\n", gpadType);
	self.type = gpadType;
end

function Controller:mapToName(btnId)
	if( not gpadinfo ) then
		error("Could not load gamepad info.", 2);
	end

	local currentSettings = gpadinfo[self.type];
	if( currentSettings ) then
		return currentSettings[btnId];
	else
		error("No settings for this gamepad type.", 2);
	end


end

function Controller:mapToKey(name)
	if( self.profile ) then
		if( self.profile[self.type] ) then
			local gpadSettings = self.profile[self.type];
			if( gpadSettings ) then
				return gpadSettings[name];
			else
				return nil; -- No binding for this button
			end
		else
			error("Cannot map key; profile does not contain info for this gamepad type.", 2);
		end
	else
		error("Cannot map key; no profile set.", 2);
	end
end

function Controller:povToKeys(angle)
	local keys = {};
	if(	(angle >= 0 and angle <= 45) or angle >= 315 ) then
		table.insert(keys, 'POV_UP');
	elseif( angle >= 135 and angle <= 225 ) then
		table.insert(keys, 'POV_DOWN');
	end

	if( angle >=45 and angle <= 135 ) then
		table.insert(keys, 'POV_RIGHT');
	elseif( angle >= 225 and angle <= 315 ) then
		table.insert(keys, 'POV_LEFT');
	end

	return table.unpack(keys);
end

function Controller:translate(btnId, action, data)
	local function povChangeSubmit(dir1, dir2)
		-- Assume all buttons are false.
		local holds = {
			['POV_UP']			=	false,
			['POV_DOWN']		=	false,
			['POV_LEFT']		=	false,
			['POV_RIGHT']		=	false,
		};

		-- Toggle on the buttons being held
		holds[dir1] = true;
		if( dir2 ) then
			holds[dir2] = true;
		end

		-- Now check 'em
		if( holds['POV_UP'] ) then
			keyboard.hold(self:mapToKey('POV_UP'));
		else
			keyboard.release(self:mapToKey('POV_UP'));
		end

		if( holds['POV_DOWN'] ) then
			keyboard.hold(self:mapToKey('POV_DOWN'));
		else
			keyboard.release(self:mapToKey('POV_DOWN'));
		end

		if( holds['POV_LEFT'] ) then
			keyboard.hold(self:mapToKey('POV_LEFT'));
		else
			keyboard.release(self:mapToKey('POV_LEFT'));
		end

		if( holds['POV_RIGHT'] ) then
			keyboard.hold(self:mapToKey('POV_RIGHT'));
		else
			keyboard.release(self:mapToKey('POV_RIGHT'));
		end
	end


	print("Translating", btnId, action);
	if( btnId == 'POV' ) then
		-- Convert POV-hat angle to 4 directions
		if( action == 'change' ) then
			-- Change our angle
			local angle = data; -- Just for convenience, so we know what data refers to in this context
			local dir1, dir2 = self:povToKeys(angle);

			povChangeSubmit(dir1, dir2);
		else
			-- Center POV
			keyboard.release(self:mapToKey('POV_UP'));
			keyboard.release(self:mapToKey('POV_DOWN'));
			keyboard.release(self:mapToKey('POV_LEFT'));
			keyboard.release(self:mapToKey('POV_RIGHT'));
		end
	else
		local name = self:mapToName(btnId);
		if( not name ) then
			error("There is no button name mapped for this gamepad type.", 2);
		end
		local vk = self:mapToKey(name);

		if( vk ) then
			if( action == 'press' ) then
				keyboard.hold(vk);
			elseif( action == 'release' ) then
				keyboard.release(vk);
			end
		end
	end
end